# $Id: DNSAgent.pm,v 2.9.4.11 2010/07/01 14:32:40 yjain Exp $ #
# $Copyrights: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

package DNSAgent;

use strict;
use warnings;
use ag_i18n_inc;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
    
);
$VERSION = '1.00';

use Carp;

my ($dig, $nsupdate);

if ( -x "$VCSHOME/bin/DNS/dig" )
{
    # Our own distribution avoiding OS dependencies
    $dig = "$VCSHOME/bin/DNS/dig";
    $nsupdate = "$VCSHOME/bin/DNS/nsupdate";
}
elsif ( -x "/usr/bin/dig" )
{
    # Location for SLES
    $dig = "/usr/bin/dig";
    $nsupdate = "/usr/bin/nsupdate";
}
elsif ( -x "/usr/sbin/dig" )
{
    # Location for RH and Solaris 10
    $dig = "/usr/sbin/dig";
    $nsupdate = "/usr/sbin/nsupdate";
}
else
{
    # dig seems to be missing
    VCSAG_LOG_MSG("E", "The dig utility could not be located, please make sure  BIND 9 software is installed", 13531);
    exit $VCSAG_RES_UNKNOWN;
}

unless ( -x $nsupdate )
{
    VCSAG_LOG_MSG("E", "The nsupdate utility could not be located, please make sure BIND 9 software is installed", 13532);
    exit $VCSAG_RES_UNKNOWN;
}


%default = (
    nameservers     => [],
    soanameserver   => '',
    domain          => '',
    errorstring     => '',
    recurse         => 1,
    retry           => 3
);

sub new {
    my ($proto, $domain) = @_;

    my $class = ref($proto) || $proto;
    my $self = { %default };
    
    Carp::croak "Domain not specified" if (!$domain && !$default{'domain'});

    $self->{'domain'} = $domain if $domain;

    bless($self, $class);

    # find authoritative name servers for this domain
    my @ans = $self->query($domain, 'SOA', 'IN');
    foreach my $tmp (@ans) {
        push(@{$self->{'nameservers'}}, $tmp->{'value'})
            if (uc($tmp->{'type'}) eq 'NS');
        $self->{'soanameserver'} = $tmp->{'value'}
            if (uc($tmp->{'type'}) eq 'SOA');
    }

    return $self;
}

sub DESTROY {}

sub nameservers {
    my $self = shift;

    if (@_) {
        my @a;
        foreach my $ns (@_) {
            if ($ns =~ /^\d+(\.\d+){0,3}$/) {
                push(@a, $ns);
            } else {
                Carp::croak "invalid nameserver specified";
            }    
        }    
        $self->{'nameservers'} = [ @a ];
    }
    return @{$self->{'nameservers'}};
}

sub recurse {
    my($self, $new_val) = @_;

    if (defined($new_val)) {
        $self->{'recurse'} = $new_val;
    }
    return $self->{'recurse'};
}

sub retry {
    my($self, $new_val) = @_;

    if (defined($new_val)) {
        $self->{'retry'} = $new_val;
    }
    return $self->{'retry'};
}

sub query {
    my ($self, $qry, $type, $class, $ns) = @_;

    my (@ans, @dig_cmd, $command);

    $type  = 'A'  unless defined($type);
    $class = 'IN' unless defined($class);

    my $dm = $self->{'domain'};
    my $tries = $self->{'retry'};

    # qualify the query with domain name if not already qualified.
    if ($qry !~ /\./) {
        $qry = "$qry"."."."$dm";
    }

    push(@dig_cmd, "$dig");
    push(@dig_cmd, "\@$ns") if $ns;
    push(@dig_cmd, "+noquestion +noadditional +nosearch");
    push(@dig_cmd, "+noauthority") unless ($type eq "SOA");
    push(@dig_cmd, "+norecurse") unless ($self->{'recurse'});
    push(@dig_cmd, "+tries=$tries") if ($tries);
    push(@dig_cmd, $qry);
    push(@dig_cmd, "-t $type");
    push(@dig_cmd, "-c $class");

    $command = join(" ", @dig_cmd);
    VCSAG_LOGDBG_MSG(6, "Running dig query: $command", "\"$command\"");
    unless (open INPUT, "$command 2>&1 |") {
        $self->{'errorstring'} = "dig failed";
        return @ans;
    }
    while (<INPUT>) {
        chomp;
        next unless (/\S+/);
        SWITCH: {
        /^;; ->>HEADER<<- / && do {
            my @fields = split(/, /, $');
            # check if status is NOERROR
            if ($fields[1] =~ /(\S+) (\S+)/) {
                unless ($2 =~ /NOERROR/) {
                    $self->{'errorstring'} = "dig error $fields[1]";
                    return @ans;
                }
            } else {
                $self->{'errorstring'} = "error parsing header $fields[1]";
                return @ans;
            }
            last SWITCH;
        };

        /^;; flags:/ && do {
            my($flags, $tmp) = split(/; /, $');
            if ($flags !~ /\baa\b/) {
                $self->{'errorstring'} = "answer not authoritative";
                return @ans;
            }
            # find the number of answers received
            my @fields = split(/, /, $tmp);    
            if ($fields[1] =~ /(\S+) (\S+)/) {
                my $num_ans = $2;
                if ($num_ans == 0) {
                    $self->{'errorstring'} = "no answer received";
                    return @ans;
                }
            } else {
                $self->{'errorstring'} = "no answer received";
                return @ans;
            }
            last SWITCH;
        };

        /^$qry/ && do {
            my @fields = split(/\s+/);
            my $resp = {};
            $resp->{'query'} = $fields[0];
            $resp->{'ttl'} = $fields[1];
            $resp->{'class'} = $fields[2];
            $resp->{'type'} = $fields[3];
            $resp->{'value'} = $fields[4];
            push(@ans, $resp);
            last SWITCH;
        };
        }
    }
    close(INPUT);
    return @ans;
}

sub update {
    my $self = shift;
    my $key_file = shift;
    my @upd = @_;
    my $tfile = "update.$$";

    Carp::croak "Empty argument list specified" unless (@upd);

    my $str = join("\n", @upd);

    unless (open(FILE, ">$tfile")) {
        Carp::croak "Unable to create temporary file ($tfile)"; 
    }

    print FILE "$str\n";
    close(FILE);

    # Call nsupdate
    if ($key_file)
    {
        VCSAG_LOGDBG_MSG(6, "Running nsupdate query: $nsupdate -k $key_file $tfile 2>&1", "$nsupdate", "$key_file", "$tfile");
        system("$nsupdate -k $key_file $tfile 2>&1");
    }
    else
    {
        VCSAG_LOGDBG_MSG(6, "Running nsupdate query: $nsupdate $tfile 2>&1", "$nsupdate", "$tfile");
        system("$nsupdate $tfile 2>&1");
    }

    # remove tfile
    unlink($tfile);
}

sub updlist {
    my $self = shift;
    my $Domain = shift;
    my $CreatePTR = shift;
    my @ResRecords = @_;
    my @UpdList;
    my @errorlist;
    my $errorlistref = \@errorlist;

    # create all update records
    VCSAG_LOGDBG_MSG(2, "Processing $#ResRecords entries", $#ResRecords);
    for (my $i = 0; $i < $#ResRecords; $i += 2)
    {
        my $j = $i + 1;
        # Remove incomplete records
        if ($ResRecords[$i] =~ /^"*$/ || $ResRecords[$j] =~ /^"*$/)
        {
            VCSAG_LOG_MSG("W", "Invalid domain name resource record ($ResRecords[$i]) maps ($ResRecords[$j])", 13533, "$ResRecords[$i]", "$ResRecords[$j]");
            next;
        }
        # Support multiple data entries per ResRecord key
        VCSAG_LOGDBG_MSG(1, "Processing entry [ ($ResRecords[$i]) => ($ResRecords[$j]) ]",
                $ResRecords[$i],
                "$ResRecords[$j]");
        foreach my $RecData (split(/\s+/, $ResRecords[$j]))
        {
            # Since a zone can only serve forward or reverse name lookups there
            # are two choices based on the $Domain - zone attribute
            if ($Domain =~ /\.ARPA\.$/i)
            {
                # reverse mapping zone all records have to be of type PTR
                ($errorlistref, @UpdList) = $self->updrecord("PTR", $Domain, 0,
                             $ResRecords[$i], $RecData, $errorlistref, @UpdList);
            }
            else
            {
                # this is forward mapping so record type can be A AAAA or CNAME
                # depending on the second value of the attribute
                if ($RecData =~ /^[0-9a-f]*(:[0-9a-f]*){1,7}/i)
                {
                    # IP V6 record

                    # The address should be noted in the possible
                    # shortest way, since named may return a different
                    # string as part of the query
                    $RecData =~ s/^0*//g;
                    $RecData =~ s/:0*/:/g;
                    my $ColDummy = $RecData;
                    $RecData =~ s/:::*/::/ 
                                unless (($ColDummy =~ s/:::*//g) > 1);
                    # Add validation for IPv6 $RecData
                    if ( $RecData !~ m/^([0-9a-f]{1,4})(\:([0-9a-f]{0,4})){1,6}\:([0-9a-f]{1,4})$/i ) {
                        VCSAG_LOG_MSG("W",
                                "Invalid IPv6 address for AAAA record [$ResRecords[$i] => $RecData]. Ignoring this entry.",
                                12038, "$ResRecords[$i]", "$RecData");
                        push(@{$errorlistref}, $RecData);
                        next;
                    }
                    VCSAG_LOGDBG_MSG(1,
                            "Assuming AAAA record for [$ResRecords[$i] => $RecData]",
                            $ResRecords[$i], $RecData);
                    ($errorlistref, @UpdList) = $self->updrecord("AAAA", $Domain,
                            $CreatePTR, $ResRecords[$i], $RecData,
                            $errorlistref, @UpdList);
                }
                elsif ($RecData =~ /^(?:22[0-3]|2[01][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])(?:\.(?:25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]|[0-9])){3}$/)
                {
                    # IP V4 record
                    VCSAG_LOGDBG_MSG(1,
                            "Assuming A record for [$ResRecords[$i] => $RecData]",
                            $ResRecords[$i], $RecData);
                    ($errorlistref, @UpdList) = $self->updrecord("A", $Domain, $CreatePTR,
                                $ResRecords[$i], $RecData, $errorlistref, @UpdList);
                }
                else
                {
                    VCSAG_LOGDBG_MSG(1,
                            "Assuming CNAME record for [$ResRecords[$i] => $RecData]",
                            $ResRecords[$i], $RecData);
                    ($errorlistref, @UpdList) = $self->updrecord("CNAME", $Domain,
                                $CreatePTR, $ResRecords[$i], $RecData,
                                $errorlistref, @UpdList);
                }
            }
        }
    }
    return ($errorlistref, @UpdList);
}


sub updrecord {
    my $self = shift;
    my $RecType = shift;
    my $Domain = shift;
    my $CreatePTR = shift;
    my $RecKey = shift;
    my $RecData = shift;
    my $errorlistref = shift;
    my @UpdList = @_;
    my ($SavedRecKey, $SavedRecData);

    # Create always fully qualified names
    $SavedRecKey = $RecKey;
    $RecKey =~ s/[^\.]$/$&\.$Domain/;
    
    # Check if names comply to RFC 1123
    if ($RecKey =~ /^.{256,}$/)
    {
        VCSAG_LOG_MSG("W", "The fully qualified name $RecKey exceeds size limit of 255 characters", 13534, "$RecKey");
        push(@{$errorlistref}, $SavedRecKey);
        return ($errorlistref, @UpdList);
    }
   unless ($RecKey =~ /^([_0-9a-z]([-_0-9a-z]{0,61}[_0-9a-z_]){0,1}\.){2,}$/i)
    {
        VCSAG_LOG_MSG("W", "The fully qualified name of record key $RecKey contains invalid characters - see RFC 1036", 13535, "$RecKey");
        push(@{$errorlistref}, $SavedRecKey);
        return ($errorlistref, @UpdList);
    }

    if ($RecType =~ /CNAME/ || $RecType =~ /PTR/)
    {
        $SavedRecData = $RecData;
        $RecData =~ s/[^\.]$/$&\.$Domain/;
        unless ($RecData =~ /^([_0-9a-z]([-_0-9a-z]{0,61}[_0-9a-z]){0,1}\.){2,}$/i)
        {
            VCSAG_LOG_MSG("W", "The fully qualified name of record data $RecData contains invalid characters - see RFC 1035", 13536, "$RecData");
            push(@{$errorlistref}, $SavedRecData);
            return ($errorlistref, @UpdList);
        }
    }

    push (@UpdList, join(" ", $Domain, $RecKey, $RecType, $RecData));
    if ($CreatePTR && $RecType =~ /^A/)
    {
        my $RevIP = "";
        if ($RecType eq "A")
        {
            $RevIP = join("\.", reverse split(/\./, $RecData));
            $RevIP = join("\.", $RevIP, "IN-ADDR.ARPA.");
        }
        elsif ($RecType eq "AAAA")
        {
            # IPv6 has up to 8 hex strings separated by ":"
            # where multiple ":" can be substitued one time by "::"
            my $Cnt = 7 - ($RecData =~ tr/://); 
            $RecData =~ s/::/":" x ($Cnt + 2)/e unless $Cnt == 0;

            # For reverse name lookups we need Nibbles (half bytes)
            for my $RevNibb (split(":", $RecData))
            {
                $Cnt = 4 - length($RevNibb);
                $RevNibb =~ s/^/"0" x $Cnt/e unless $Cnt == 0;
                $RevNibb = reverse $RevNibb;
                $RevNibb =~ s/./$&\./g;
                $RevIP = "$RevNibb"."$RevIP";
            }
            $RevIP = "$RevIP"."IP6.ARPA.";
	    # The use of IP6.INT is depreciated see RFC 4159 August 2005
            # $RevIP = "$RevIP"."IP6.INT.";
        }
        ($errorlistref, @UpdList) = $self->updrecord("PTR", "", 0, $RevIP,
                                     $RecKey, $errorlistref, @UpdList);

    }
    return ($errorlistref, @UpdList);
}

sub upddelent {
    my $self = shift;
    my $NServer = shift;
    my $RecKey = shift;
    my $RecType = shift;

    $self->recurse(0);

    my @Answer = $self->query($RecKey, $RecType, 'IN', $NServer);
    if (@Answer)
    {
         foreach my $tmp (@Answer)
         {
             return 1 if (uc($tmp->{'type'}) eq $RecType);
         }
     }
     return 0;
}

sub ping {
    my $self = shift;
    my $master = shift;

    `ping $master -w10 -c2 2> /dev/null`;
    return $?;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# documentation for module.

=head1 NAME

DNSAgent - Perl class for querying and updating name servers 

=head1 SYNOPSIS

use DNSAgent;

=head1 DESCRIPTION

Instances of the DNSAgent class represent DNS resolver objects. The DNSAgent
class uses 'dig' and 'nsupdate' to query and update the nameserver respectively.

=head1 METHODS

=cut

=head2 new

    $res = DNSAgent->new("example.com");

Creates a new DNS resolver object.

=cut

=head2 query

    $ans = $res->query('www', 'CNAME', 'IN', '192.152.1.1');

Performs a DNS query for the given name, with the specified name server (last
argument). If nameserver is not specified, the query is send to the default
nameserver.

=cut

=head2 update

    push(@upd, "server nameserver1.example.com");
    push(@upd, "zone www.example.com");
    push(@upd, "update add www.veritas.com 86400 CNAME somehost.example.com"); 
    $res->update(@upd);

Dynamically update nameserver.

=cut

=head1 AUTHOR

Uli Maurus, support@veritas.com 

=head1 COPYRIGHT

 $Copyright: Copyright (c) 2011 Symantec Corporation.
 All rights reserved.

 THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.

 The Licensed Software and Documentation are deemed to be commercial
 computer software as defined in FAR 12.212 and subject to restricted
 rights as defined in FAR Section 52.227-19 "Commercial Computer
 Software - Restricted Rights" and DFARS 227.7202, "Rights in
 Commercial Computer Software or Commercial Computer Software
 Documentation", as applicable, and any successor regulations. Any use,
 modification, reproduction release, performance, display or disclosure
 of the Licensed Software and Documentation by the U.S. Government
 shall be solely in accordance with the terms of this Agreement.  $

=head1 SEE ALSO

perl(1)
dig(1)
nsupdate(8)

=cut
