# $Id: net_functions.pl,v 2.8.2.3 2010/07/01 15:08:04 yjain Exp $ #
# $Copyrights: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

use strict;
use warnings;
use Fcntl qw(:flock);
use Socket qw(inet_aton inet_ntoa);
use POSIX;

use ag_i18n_inc;

my $vcs_home = $VCSHOME;
my $vcs_log = $VCSLOG;

my ($ifconfig, $cut, $grep, $ping, $ip) =
    ("ifconfig", "cut", "grep", "ping", "ip");

#
# checks for validity of ip address.
# input : Two arguments 
#   1. ip address
#   2. ret val in which value will be return
#
# output : Returns the same input parameters
#   1. updated ipaddres
#   2. return error/success value
#      values = 0  address is invalid
#             = 1  address is valid
#             = 2  empty address specified
#
# function :
#   1. removes extra spaces from ip address
#   2. removes starting zeroes from the ip address i.e. 00001 = 1 or 00000 = 0
#   3. checks whether ip address is in .(dot) notation also contains only digits
#      in 0,255 range also it is not a loopback address
#   4. checks is ip address is not an empty string

sub validate_ip_address
{
    my (@input) = @_;

    #input[0] is address and input[1] is ret val.
    #return 0 for address invalidity, 1 for invalidity and 2 for empty address

    my (@ipaddr, $i, $ret_val);
    $input[0] =~ s/^\s+//;
    $input[0] =~ s/\s+//g;
    $input[0] =~ s/\s+$//;

    if(!length($input[0]))
    {
        $input[1] = 2;
        return(@input);
    }

    @ipaddr = split(/\./, $input[0]);
    if($#ipaddr != 3)
    {
           $input[1] = 0;
           return(@input);
    }

    for($i=0; $i<=$#ipaddr; $i++)
    {
       if($ipaddr[$i] !~ /^\d+$/)
       {
           $input[1] = 0;
           return(@input);
       }
       else
       {
           if($ipaddr[$i] eq "" || $ipaddr[$i] < 0 || $ipaddr[$i] > 255)
           {
              $input[1] = 0;
              return(@input);
           }
           else
           {
               $ipaddr[$i] =~ s/^0+//;
               if($ipaddr[$i] eq "")
               {
                   $ipaddr[$i] = 0;
               }
           }
       }
    }

    $input[0] = join(".", @ipaddr);

    if(($ipaddr[0] eq 127 && $ipaddr[1] eq 0 &&
        $ipaddr[2] eq 0 && $ipaddr[3] eq 1) ||
       ($ipaddr[0] eq 0  && $ipaddr[1] eq 0 &&
        $ipaddr[2] eq 0 && $ipaddr[3] eq 0))
    {
        $input[1] = 0;
        return(@input);
    }

    $input[1] = 1;
    return(@input);
}

sub find_alias
{
	my ($dev, $addr, $ret);
	($dev, $addr) = @_;
	$ret = "";
	if ($dev eq "") {
		return $ret;
	}
	open IPCMD, "$ip addr show dev $dev|";
	foreach (<IPCMD>) {
		if (! / $addr\//) {
			next;
		}
		my @outp_flds = split;
		$ret = $outp_flds[-1];
	}
	close IPCMD;
	return $ret;
}

#
# remove (down) ip address which is assigned to one of the aliases of the given
# device
#
# input : 
#   1. ip address 
#   2. device
#
# output : returns 
#   1. 0 if the specified device alias is really down
#   2. 1 if failed to down the ip address

sub remove_ip
{
    my ($address, $dev, $actual_dev, $ret_val);
    $address = $_[0];
    $dev = $_[1];
    $actual_dev = find_alias($dev, $address);
    
    if(length($actual_dev))
    {
        `$vcs_home/bin/haifconfig -d $actual_dev`;
        `$vcs_home/bin/haifconfig -s $actual_dev -a $address`;
        $ret_val = $?/256;
        if($ret_val == 1)
        {
            return 1;
        }
        else
        {
            return 0;
        }
    }
}

#
# routine to get the next alias number of the interface
#
# input :
#   1. device of which available alias number is to be found
#
# output :
#   1. returns next available alias number

sub getaliasnum
{
    my($alias,$device);
    $device = $_[0];

    $alias = `$vcs_home/bin/haifconfig -a $device`;
    $alias = 0 unless defined $alias;
    chomp ($alias);

    return($alias);
}


#
# to check if ip address is already configured elsewhere on the network
# 
# input :
#   1. ip address that need to be checked
#
# output : returns 
#   1. 1 if ping returns success means ip address is already configured
#   2. 0 if not  configured means ping to that ip address failed
#

sub is_ip_configured
{
    my ($ip, $ret_val);
    $ip = $_[0];
    
    `$ping $ip -c 2 -w 10 2> /dev/null`;
    $ret_val = $?;
    
    if($ret_val eq 0)
    {
        return 1;
    }

    return 0;
}

#
# to read the received packet count of the interface stored in a file
#
# input :
#   1. dev_stat_file name in which count is stored.
#
# output : 
#   1. returns received packet count stored by get_current_stats routine
# 

sub get_prev_stats
{
    my($nic_stats, $dev_stat_file);
    $nic_stats = 0;
    $dev_stat_file = $_[0];
    
    if ( -f "$dev_stat_file")
    {
        open (STATS, "$dev_stat_file");
        while (<STATS>)
        {
            $nic_stats = $_;
        }
        close (STATS);
    }

    return $nic_stats;
}

#
# to read the received packet count of the interface iand store it in a file
#
# input :
#   1. device name of which statistices is to be read/stored.
#   2. dev_stat_file name in which count is to be stored.
#
# output :
#   1. returns received packet count
#

sub get_current_stats
{
    my($nic_stats, $devname, $dev_stat_file);
    $nic_stats = 0;
    $devname = $_[0];
    $dev_stat_file = $_[1];

    $nic_stats = `$vcs_home/bin/haifconfig -r $devname`;
    
    open(NICSTATS, ">$dev_stat_file");
    print NICSTATS $nic_stats;
    close(NICSTATS);

    return ($nic_stats);
}

#
# to do ping test i.e. to ping the network hosts to check the activity of the
# interface/device
#
# input :
#   1. device name of which activity is to be determine.
#   2. network host list to do ping from the given device
#
# output : returns
#   1. 0 if ping to any of the network host succeeds
#   2. 1 if ping to all of the network host fails
#

sub ping_networkhost_test
{
    my($ret_val, $i, $device, $numhosts, $hostslistref);
    $ret_val = 1;
    $device = $_[0];
    $hostslistref = $_[1];

    #print "\n    join(' ',@{$hostslistref}) \n";    To get the array

    $numhosts=$#{$hostslistref}; # total no of array elements

    #Test if "ANY" of the hosts in the list
    #are reachable. Send a message to the log
    #file if a host is not reachable

    for($i=0; $i<=$numhosts; $i++)
    {
        `$ping -I $device $hostslistref->[$i] -w10 -c2 2> /dev/null`;
        $ret_val=$?;
        if($ret_val eq 0)
        {
            return 0;
        }
    }
 
    #Return 1 if test fails i.e. none of the n/w hosts are reachable
    return 1;
}

#
# to get lock on a file
#
# input : 
#   1. lock file name
#   2. flag to indicate the lock type
#         0 read lock
#         1 write lock
#
# output : 
#   1. returns file descriptor
#

sub acquire_lock
{
    my($arg, $ret_val, $file);
    $ret_val = 0;
    $file = $_[0];
    
    if($_[1] eq 1)
    {
        open (FILE, ">$file"); #write lock
        $arg = pack("SSLLLLLLLL", F_WRLCK(), 0,0,0,0,0,0,0,0,0);
    }
    else
    {
        open(FILE, "<$file"); #read lock
        $arg = pack("SSLLLLLLLL", F_RDLCK(), 0,0,0,0,0,0,0,0,0);
    }

    $ret_val = fcntl(FILE, F_SETLKW(), $arg);
    return \*FILE;
}

#
# routine to release lock
#
# input : file descriptor
#

sub release_lock
{
    close($_[0]);
}

sub get_nmask
{
       my @mask_line;
       my ($dev);

       $dev = shift;
       open IFCFG, "$ifconfig $dev|";
       while (<IFCFG>) {
               if (! /Mask:/ ) {
                       # Skip lines without the Mask
                       next;
               }
               close IFCFG;
               # Last element after a ':'
               @mask_line = split (':', $_);
               chomp($mask_line[-1]);
               return $mask_line[-1];
       }
       close IFCFG;
       return undef;
}

sub get_network
{
    my ($ipaddress, $netmask) = @_;
    my ($network) = ("");

    $ipaddress = inet_aton ($ipaddress);
    $netmask = inet_aton ($netmask);
    $network = $ipaddress & $netmask;
    $network = inet_ntoa ($network);

    return $network;
}
