
use strict;
use warnings;
use POSIX;

use ag_i18n_inc;

my $vcs_home = $VCSHOME;
my $vcs_log = $VCSLOG;

# PIDFILE_OLDTYPE -> pidfile will be always named smbd.pid
# PIDFILE_NEWTYPE -> pidfile will be smbd.pid for standard conf file
#                 -> pidfile will be smbd-<conffile>.pid for non-standard conf file

use constant PIDFILE_OLDTYPE => 0;
use constant PIDFILE_NEWTYPE => 1;

my ($smbd, $PS) = ("smbd", "ps");

# 
# derive pid file path from samba Configuration file, lock directory and daemon
#

sub get_pid_file_path
{
	my ($res_name, $smb_conf_path, $lock_dir, $which_daemon ) = (@_);
	my ($default_conffile,$pid_file_path) = ("", "");
	my ($pid_extn) = ".pid";
	my $default_conffile_path = "$vcs_log/lock/default_conffile_path";
	if( -f $default_conffile_path )
	{
		if (open (DEFCONFFILE, $default_conffile_path))
		{
			$default_conffile = <DEFCONFFILE>;
			$default_conffile = "" unless (defined $default_conffile);
			chomp ( $default_conffile);
			close(DEFCONFFILE);
		}
	} 

	my $pidfile_behavior = PIDFILE_OLDTYPE;
	my $pidfile_behavior_file = "$vcs_log/lock/$res_name"."_smbd_pidfile_behavior";
	if (-f $pidfile_behavior_file)
	{
		if (open (PIDFILETYPE, $pidfile_behavior_file))
		{
			$pidfile_behavior = <PIDFILETYPE>;
			$pidfile_behavior = PIDFILE_OLDTYPE unless (defined $pidfile_behavior);
			chomp ($pidfile_behavior);
			close (PIDFILETYPE);
		}
	} 

	# Derive the pid file name from ConfFile and LockDir
	if(not defined($default_conffile) or $default_conffile eq "")
	{ 
		$default_conffile = `smbd -b 2>/dev/null | grep CONFIGFILE | awk '{print \$2}'`;
		if (not ($default_conffile)) {
			$default_conffile = $smb_conf_path; 
			VCSAG_LOGDBG_MSG(5, "Failed to fetch default conffile, using [$smb_conf_path] as default");
		} else {
			VCSAG_LOGDBG_MSG(5, "Default conffile value of smbd is [$default_conffile]");
		}
		chomp ($default_conffile);
	}
	if(($pidfile_behavior eq PIDFILE_OLDTYPE) or ($smb_conf_path eq $default_conffile))
	{
		$pid_file_path = "$lock_dir/$which_daemon.pid";
	}
	else
	{
		$smb_conf_path =~ s/(.*\/)(.*)$/$2/;
		$pid_file_path = "$lock_dir/$which_daemon-".$smb_conf_path.$pid_extn;
	}
	return $pid_file_path;
}

#
# returns running smbd interfaces for a given smbd pid by reading from / writing to 
# share_<share_name> file
#

sub get_smbd_interfaces
{
	my ($share_name, $pid) = (@_);
	my ($vcsLock) = "$vcs_log/lock";
	#match pid from pidfile with interface file entry
	my ($share_file) = $vcsLock."/"."share_".$share_name;
	my ($share_file_pid, $share_file_interfaces) = ("", ""); 
	if( -f $share_file)
	{
		if(open(SHAREFILE, $share_file))
		{
			my $share_file_contents = do { local $/; <SHAREFILE> };
			($share_file_pid, $share_file_interfaces) = split(/\n/,$share_file_contents); 
			close(SHAREFILE);
			$share_file_pid = "" unless( defined $share_file_pid );
			$share_file_interfaces = "" unless( defined $share_file_interfaces );
			chomp($share_file_pid);
			chomp($share_file_interfaces);
			VCSAG_LOGDBG_MSG("1", "share file pid is: $share_file_pid and interfaces are: $share_file_interfaces ", "$share_file_pid", "$share_file_interfaces");
		}
	}

	if(not (-f $share_file) or not ($share_file_pid eq $pid) or not ($share_file_interfaces))
	{
		if(open(SHAREFILE, ">".$share_file))
		{
			$share_file_pid = $pid;
			$share_file_interfaces = `netstat -lnp 2>/dev/null | grep $pid/$smbd | awk '{print \$4}' | cut -d: -f1 | sort | uniq | xargs`;
			$share_file_interfaces = "" unless defined ($share_file_interfaces);
			chomp($share_file_interfaces);
			if ($share_file_interfaces eq "")
			{
				$share_file_interfaces = "127.0.0.1"; 
			}
			my $write_pos = 0;
			seek SHAREFILE, $write_pos, 0;
			print (SHAREFILE $share_file_pid."\n".$share_file_interfaces);
			$write_pos = tell SHAREFILE;
			truncate SHAREFILE, $write_pos;
			close(SHAREFILE);
			VCSAG_LOGDBG_MSG("1", "pid : $share_file_pid and interfaces : $share_file_interfaces written to share file", "$share_file_pid", "$share_file_interfaces");
		}
	}
	$share_file_interfaces = "" unless defined ($share_file_interfaces);
	chomp($share_file_interfaces);
	my @interfaces_array = split(/ /,$share_file_interfaces);
	return @interfaces_array;
}

#
# kill all child pids for a given pid
#

sub kill_child_pids
{
	my ($pid) = (@_);
	if(open(CHILDPID, "$PS -C $smbd -o pid,ppid --no-header 2>/dev/null | awk \'{ if ( \$2 == $pid ) { print \$1 }}\' | ") )
	{
		while(<CHILDPID>)
		{
			chomp($_);
			kill 9, $_;
		}
		close(CHILDPID);
	}
	else
	{
		VCSAG_LOG_MSG ("E", "Unable to get output of $PS command.", 10508, "$PS");
		exit 1; # returning failure
	}
}

#
# delete share_<share_name> file
#

sub unlink_sharefile
{
	my ($share_name) = (@_);
	my ($vcsLock) = "$vcs_log/lock";
	my ($share_file) = $vcsLock."/"."share_".$share_name;
	unlink ($share_file);
}

